// Illustrations file:
//
// I sometimes want to display the Simblob map with certain types of
// constructs to graphically illustrate specific situations.
// Examples: hexagonal grid structure, triangular grid structure,
// rhomboidal grid structure, rivers, walls, mountains, valleys,
// lakes, various camera viewpoints, pathfinding.
//
// This file sets up specific examples for rendering; I then take
// screenshots of them.

#include "include_gl.h"

#include "illustrations.h"
#include "rendering.h"

namespace Illustrations {

  void hexagonal_grid() {
    glPolygonOffset(1.0, 2.0);

    glColor4f(0.3f, 0.3f, 0.6f, 1.0f);
    glBegin(GL_QUADS);
    glVertex3f(Map::min_x, Map::min_y, 0.0f);
    glVertex3f(Map::max_x, Map::min_y, 0.0f);
    glVertex3f(Map::max_x, Map::max_y, 0.0f);
    glVertex3f(Map::min_x, Map::max_y, 0.0f);
    glEnd();
    
    for (int m = 0; m < Map::size; ++m) {
      for (int n = 0; n < Map::size; ++n) {
        glColor4f(0.7f, 0.7f, 0.7f, 1.0f);
        Rendering::draw_solid_hexagon(m, n, Map::height);
      }
    }
    glPolygonOffset(1.0, 0.0);
    
    for (int m = 0; m < Map::size; ++m) {
      for (int n = 0; n < Map::size; ++n) {
        if (m == 0 || n == 0)
          glColor4f(1.0f, 0.5f, 0.5f, 1.0f);
        else if (m == Map::size/2)
          glColor4f(0.5f, 0.5f, 1.0f, 1.0f);
        else if (n == Map::size/2)
          glColor4f(0.5f, 1.0f, 0.5f, 1.0f);
        else
          glColor4f(0.5f, 0.5f, 0.6f, 1.0f);
        Rendering::draw_outline_hexagon(m, n, Map::height);
      }
    }
  }
}
