#ifndef _POINTS3_H
#define _POINTS3_H

#include <cmath>

struct Vector3d;

struct Point3d {
  float x, y, z;
  Point3d(): x(0), y(0), z(0) {}
  Point3d(float x_, float y_, float z_): x(x_), y(y_), z(z_) {}

  Point3d& operator += (const Vector3d& v);
};

struct Vector3d {
  float x, y, z;
  Vector3d(): x(0), y(0), z(0) {}
  Vector3d(float x_, float y_, float z_): x(x_), y(y_), z(z_) {}
  float length_squared() const {
    return x*x + y*y + z*z;
  }
  float length() const {
    return sqrt(length_squared());
  }
  
  Vector3d& operator += (const Vector3d& v);
  Vector3d& operator -= (const Vector3d& v);
  Vector3d& operator *= (float t);
};

inline Vector3d& Vector3d::operator *= (float t) {
  x *= t;
  y *= t;
  z *= t;
  return *this;
}

inline Vector3d operator * (const Vector3d& v, float t) {
  return Vector3d(v.x*t, v.y*t, v.z*t);
}

inline Vector3d operator * (float t, const Vector3d& v) {
  return Vector3d(v.x*t, v.y*t, v.z*t);
}

inline Vector3d operator / (const Vector3d& v, float t) {
  float inv_t = 1.0f/t;
  return Vector3d(v.x*inv_t, v.y*inv_t, v.z*inv_t);
}

inline Vector3d& Vector3d::operator += (const Vector3d& v) {
  x += v.x;
  y += v.y;
  z += v.z;
  return *this;
}

inline Vector3d& Vector3d::operator -= (const Vector3d& v) {
  x -= v.x;
  y -= v.y;
  z -= v.z;
  return *this;
}

inline Vector3d operator + (const Vector3d& v, const Vector3d& w) {
  return Vector3d(v.x+w.x, v.y+w.y, v.z+w.z);
}

inline Vector3d operator - (const Vector3d& v, const Vector3d& w) {
  return Vector3d(v.x-w.x, v.y-w.y, v.z-w.z);
}

inline Point3d& Point3d::operator += (const Vector3d& v) {
  x += v.x;
  y += v.y;
  z += v.z;
  return *this;
}

inline Point3d operator + (const Vector3d& v, const Point3d& p) {
  return Point3d(v.x+p.x, v.y+p.y, v.z+p.z);
}

inline Point3d operator + (const Point3d& p, const Vector3d& v) {
  return Point3d(p.x+v.x, p.y+v.y, p.z+v.z);
}

inline Point3d operator - (const Point3d& p, const Vector3d& v) {
  return Point3d(p.x-v.x, p.y-v.y, p.z-v.z);
}

inline Vector3d operator - (const Point3d& p, const Point3d& q) {
  return Vector3d(p.x-q.x, p.y-q.y, p.z-q.z);
}

inline Vector3d normalize(const Vector3d& v) {
  float length = v.length();
  float d = (length == 0.0f)? 1.0f : 1.0f / length;
  return v * d;
}

inline float dot(const Vector3d& v, const Vector3d& w) {
  return v.x*w.x + v.y*w.y + v.z*w.z;
}

inline float dist(const Point3d& p, const Point3d& q) {
  return (p-q).length();
}

inline Vector3d cross(const Vector3d& a, const Vector3d& b) {
  return Vector3d(
    a.y*b.z - a.z*b.y,
    a.z*b.x - a.x*b.z,
    a.x*b.y - a.y*b.x
    );
}

#endif
