#ifndef _SIMULATE_H
#define _SIMULATE_H

#include "world.h"

/* Game map */
namespace Map {
  extern bool flooding; // is there massive flooding going on?
  
  extern array moisture; // how moist is the land?
  extern array vegetation; // how much is growing there
  
  extern array water;
  extern array outflow; // how much water flowed out last time
  
  /* We need the water momentum for erosion algorithms.  However,
  Vector3d is pretty heavyweight (at three doubles), and we could
  probably get away with just using two bytes or two shorts. This is also
  used for the texture animation.  */
  extern Vector3d water_momentum[size][size];

  /* We need to keep track of the silt in the water.  It affects
     coloring, but more importantly it affects erosion.  As water
     moves fast, it can hold a lot of silt.  As it slows down, it
     dumps silt. */
  extern array silt;
     
  /* Water sources */
  struct WaterSource {
    int m, n;
    float frequency;
    WaterSource(int m_, int n_, float frequency_)
      : m(m_), n(n_), frequency(frequency_) {}
  };

  extern std::vector<WaterSource> water_sources;

  void setup_map();
  void simulate();
}

/* The traversal scheme is to visit every other row, every other
   column.  This guarantees that we won't visit two adjacent nodes in
   the same pass.  However .. it is still possible that one water flow
   operation will affect another one (two hexes away). */
#define LOOP1(M,N,_phase) for (int M = (_phase&1); M < Map::size; M += 2) for (int N = ((_phase&2) >> 1); N < Map::size; N += 2)
#define LOOP(M,N) for (int _phase_start=random()%4, _phase = _phase_start; _phase < 4+_phase_start; ++_phase) LOOP1(M,N,_phase)

#endif
